// Chuck Benz, Hollis, NH   Copyright (c)2002
//
// The information and description contained herein is the
// property of Chuck Benz.
//
// Permission is granted for any reuse of this information
// and description as long as this copyright notice is
// preserved.  Modifications may be made as long as this
// notice is preserved.

// per Widmer and Franaszek

`timescale 1ns / 1ps

module decode (datain, dispin, dataout, dispout, code_err, disp_err) ;
  input [9:0]   datain ;
  input		dispin ;
  output [8:0]	dataout ;
  output	dispout ;
  output	code_err ;
  output	disp_err ;

  wire wai = datain[0] ;
  wire wbi = datain[1] ;
  wire wci = datain[2] ;
  wire wdi = datain[3] ;
  wire wei = datain[4] ;
  wire wii = datain[5] ;
  wire wfi = datain[6] ;
  wire wgi = datain[7] ;
  wire whi = datain[8] ;
  wire wji = datain[9] ;

  wire aeqb = (wai & wbi) | (!wai & !wbi) ;
  wire ceqd = (wci & wdi) | (!wci & !wdi) ;
  wire p22 = (wai & wbi & !wci & !wdi) |
	     (wci & wdi & !wai & !wbi) |
	     ( !aeqb & !ceqd) ;
  wire p13 = ( !aeqb & !wci & !wdi) |
	     ( !ceqd & !wai & !wbi) ;
  wire p31 = ( !aeqb & wci & wdi) |
	     ( !ceqd & wai & wbi) ;

  wire p40 = wai & wbi & wci & wdi ;
  wire p04 = !wai & !wbi & !wci & !wdi ;

  wire disp6a = p31 | (p22 & dispin) ; // pos disp if p22 and was pos, or p31.
   wire disp6a2 = p31 & dispin ;  // disp is ++ after 4 bits
   wire disp6a0 = p13 & ! dispin ; // -- disp after 4 bits

  wire disp6b = (((wei & wii & ! disp6a0) | (disp6a & (wei | wii)) | disp6a2 |
		  (wei & wii & wdi)) & (wei | wii | wdi)) ;

  // The 5B/6B decoding special cases where ABCDE != abcde

  wire p22bceeqi = p22 & wbi & wci & (wei == wii) ;
  wire p22bncneeqi = p22 & !wbi & !wci & (wei == wii) ;
  wire p13in = p13 & !wii ;
  wire p31i = p31 & wii ;
  wire p13dei = p13 & wdi & wei & wii ;
  wire p22aceeqi = p22 & wai & wci & (wei == wii) ;
  wire p22ancneeqi = p22 & !wai & !wci & (wei == wii) ;
  wire p13en = p13 & !wei ;
  wire anbnenin = !wai & !wbi & !wei & !wii ;
  wire abei = wai & wbi & wei & wii ;
  wire cdei = wci & wdi & wei & wii ;
  wire cndnenin = !wci & !wdi & !wei & !wii ;

  // non-zero disparity cases:
  wire p22enin = p22 & !wei & !wii ;
  wire p22ei = p22 & wei & wii ;
  //wire p13in = p12 & !wii ;
  //wire p31i = p31 & wii ;
  wire p31dnenin = p31 & !wdi & !wei & !wii ;
  //wire p13dei = p13 & wdi & wei & wii ;
  wire p31e = p31 & wei ;

  wire compa = p22bncneeqi | p31i | p13dei | p22ancneeqi |
		p13en | abei | cndnenin ;
  wire compb = p22bceeqi | p31i | p13dei | p22aceeqi |
		p13en | abei | cndnenin ;
  wire compc = p22bceeqi | p31i | p13dei | p22ancneeqi |
		p13en | anbnenin | cndnenin ;
  wire compd = p22bncneeqi | p31i | p13dei | p22aceeqi |
		p13en | abei | cndnenin ;
  wire compe = p22bncneeqi | p13in | p13dei | p22ancneeqi |
		p13en | anbnenin | cndnenin ;

  wire wao = wai ^ compa ;
  wire wbo = wbi ^ compb ;
  wire wco = wci ^ compc ;
  wire wdo = wdi ^ compd ;
  wire weo = wei ^ compe ;

  wire feqg = (wfi & wgi) | (!wfi & !wgi) ;
  wire heqj = (whi & wji) | (!whi & !wji) ;
  wire fghj22 = (wfi & wgi & !whi & !wji) |
		(!wfi & !wgi & whi & wji) |
		( !feqg & !heqj) ;
  wire fghjp13 = ( !feqg & !whi & !wji) |
		 ( !heqj & !wfi & !wgi) ;
  wire fghjp31 = ( (!feqg) & whi & wji) |
		 ( !heqj & wfi & wgi) ;

  wire dispout = (fghjp31 | (disp6b & fghj22) | (whi & wji)) & (whi | wji) ;

  wire wko = ( (wci & wdi & wei & wii) | ( !wci & !wdi & !wei & !wii) |
		(p13 & !wei & wii & wgi & whi & wji) |
		(p31 & wei & !wii & !wgi & !whi & !wji)) ;

  wire alt7 =   (wfi & !wgi & !whi & // 1000 cases, where disp6b is 1
		 ((dispin & wci & wdi & !wei & !wii) | wko |
		  (dispin & !wci & wdi & !wei & !wii))) |
		(!wfi & wgi & whi & // 0111 cases, where disp6b is 0
		 (( !dispin & !wci & !wdi & wei & wii) | wko |
		  ( !dispin & wci & !wdi & wei & wii))) ;

  wire k28 = (wci & wdi & wei & wii) | ! (wci | wdi | wei | wii) ;
  // k28 with positive disp into fghi - .1, .2, .5, and .6 special cases
  wire k28p = ! (wci | wdi | wei | wii) ;
  wire wfo = (wji & !wfi & (whi | !wgi | k28p)) |
	    (wfi & !wji & (!whi | wgi | !k28p)) |
	    (k28p & wgi & whi) |
	    (!k28p & !wgi & !whi) ;
  wire wgo = (wji & !wfi & (whi | !wgi | !k28p)) |
	    (wfi & !wji & (!whi | wgi |k28p)) |
	    (!k28p & wgi & whi) |
	    (k28p & !wgi & !whi) ;
  wire who = ((wji ^ whi) & ! ((!wfi & wgi & !whi & wji & !k28p) | (!wfi & wgi & whi & !wji & k28p) |
			    (wfi & !wgi & !whi & wji & !k28p) | (wfi & !wgi & whi & !wji & k28p))) |
	    (!wfi & wgi & whi & wji) | (wfi & !wgi & !whi & !wji) ;

  wire disp6p = (p31 & (wei | wii)) | (p22 & wei & wii) ;
  wire disp6n = (p13 & ! (wei & wii)) | (p22 & !wei & !wii) ;
  wire disp4p = fghjp31 ;
  wire disp4n = fghjp13 ;

  assign code_err = p40 | p04 | (wfi & wgi & whi & wji) | (!wfi & !wgi & !whi & !wji) |
		    (p13 & !wei & !wii) | (p31 & wei & wii) |
		    (wei & wii & wfi & wgi & whi) | (!wei & !wii & !wfi & !wgi & !whi) |
		    (wei & !wii & wgi & whi & wji) | (!wei & wii & !wgi & !whi & !wji) |
		    (!p31 & wei & !wii & !wgi & !whi & !wji) |
		    (!p13 & !wei & wii & wgi & whi & wji) |
		    (((wei & wii & !wgi & !whi & !wji) |
		      (!wei & !wii & wgi & whi & wji)) &
		     ! ((wci & wdi & wei) | (!wci & !wdi & !wei))) |
		    (disp6p & disp4p) | (disp6n & disp4n) |
		    (wai & wbi & wci & !wei & !wii & ((!wfi & !wgi) | fghjp13)) |
		    (!wai & !wbi & !wci & wei & wii & ((wfi & wgi) | fghjp31)) |
		    (wfi & wgi & !whi & !wji & disp6p) |
		    (!wfi & !wgi & whi & wji & disp6n) |
		    (wci & wdi & wei & wii & !wfi & !wgi & !whi) |
		    (!wci & !wdi & !wei & !wii & wfi & wgi & whi) ;

  assign dataout = {wko, who, wgo, wfo, weo, wdo, wco, wbo, wao} ;

  // my disp err fires for any legal codes that violate disparity, may fire for illegal codes
   assign disp_err = ((dispin & disp6p) | (disp6n & !dispin) |
		      (dispin & !disp6n & wfi & wgi) |
		      (dispin & wai & wbi & wci) |
		      (dispin & !disp6n & disp4p) |
		      (!dispin & !disp6p & !wfi & !wgi) |
		      (!dispin & !wai & !wbi & !wci) |
		      (!dispin & !disp6p & disp4n) |
		      (disp6p & disp4p) | (disp6n & disp4n)) ;

endmodule
