// This file : par16prbs.v
// Rev 2.0   : 02/23/2024


`timescale 1ns/1ps

module par16prbs (
  output wire [15:0] QOUT,
  input  wire [1:0] PSEL,
  input  wire INVD,
  input  wire RST,
  input  wire CLK,
  input  wire PCC,
  input  wire PSS
);

  wire [15:0] d07p;
  wire [15:0] d15p;
  wire [15:0] d23p;
  wire [15:0] d31p;

  wire [31:0] dnext;

  reg  [31:0] dbuf;

  assign d07p[0]  = dbuf[25]  ~^  dbuf[26];
  assign d07p[1]  = dbuf[26]  ~^  dbuf[27];
  assign d07p[2]  = dbuf[27]  ~^  dbuf[28];
  assign d07p[3]  = dbuf[28]  ~^  dbuf[29];
  assign d07p[4]  = dbuf[29]  ~^  dbuf[30];
  assign d07p[5]  = dbuf[30]  ~^  dbuf[31];
  assign d07p[6]  = dbuf[31]  ~^  d07p[0];
  assign d07p[7]  = d07p[0]   ~^  d07p[1];
  assign d07p[8]  = d07p[1]   ~^  d07p[2];
  assign d07p[9]  = d07p[2]   ~^  d07p[3];
  assign d07p[10] = d07p[3]   ~^  d07p[4];
  assign d07p[11] = d07p[4]   ~^  d07p[5];
  assign d07p[12] = d07p[5]   ~^  d07p[6];
  assign d07p[13] = d07p[6]   ~^  d07p[7];
  assign d07p[14] = d07p[7]   ~^  d07p[8];
  assign d07p[15] = d07p[8]   ~^  d07p[9];

  assign d15p[0]  = dbuf[17]  ~^  dbuf[18];
  assign d15p[1]  = dbuf[18]  ~^  dbuf[19];
  assign d15p[2]  = dbuf[19]  ~^  dbuf[20];
  assign d15p[3]  = dbuf[20]  ~^  dbuf[21];
  assign d15p[4]  = dbuf[21]  ~^  dbuf[22];
  assign d15p[5]  = dbuf[22]  ~^  dbuf[23];
  assign d15p[6]  = dbuf[23]  ~^  dbuf[24];
  assign d15p[7]  = dbuf[24]  ~^  dbuf[25];
  assign d15p[8]  = dbuf[25]  ~^  dbuf[26];
  assign d15p[9]  = dbuf[26]  ~^  dbuf[27];
  assign d15p[10] = dbuf[27]  ~^  dbuf[28];
  assign d15p[11] = dbuf[28]  ~^  dbuf[29];
  assign d15p[12] = dbuf[29]  ~^  dbuf[30];
  assign d15p[13] = dbuf[30]  ~^  dbuf[31];
  assign d15p[14] = dbuf[31]  ~^  d15p[0];
  assign d15p[15] = d15p[0]   ~^  d15p[1];

  assign d23p[0]  = dbuf[9]   ~^  dbuf[14];
  assign d23p[1]  = dbuf[10]  ~^  dbuf[15];
  assign d23p[2]  = dbuf[11]  ~^  dbuf[16];
  assign d23p[3]  = dbuf[12]  ~^  dbuf[17];
  assign d23p[4]  = dbuf[13]  ~^  dbuf[18];
  assign d23p[5]  = dbuf[14]  ~^  dbuf[19];
  assign d23p[6]  = dbuf[15]  ~^  dbuf[20];
  assign d23p[7]  = dbuf[16]  ~^  dbuf[21];
  assign d23p[8]  = dbuf[17]  ~^  dbuf[22];
  assign d23p[9]  = dbuf[18]  ~^  dbuf[23];
  assign d23p[10] = dbuf[19]  ~^  dbuf[24];
  assign d23p[11] = dbuf[20]  ~^  dbuf[25];
  assign d23p[12] = dbuf[21]  ~^  dbuf[26];
  assign d23p[13] = dbuf[22]  ~^  dbuf[27];
  assign d23p[14] = dbuf[23]  ~^  dbuf[28];
  assign d23p[15] = dbuf[24]  ~^  dbuf[29];

  assign d31p[0]  = dbuf[1]   ~^  dbuf[4];
  assign d31p[1]  = dbuf[2]   ~^  dbuf[5];
  assign d31p[2]  = dbuf[3]   ~^  dbuf[6];
  assign d31p[3]  = dbuf[4]   ~^  dbuf[7];
  assign d31p[4]  = dbuf[5]   ~^  dbuf[8];
  assign d31p[5]  = dbuf[6]   ~^  dbuf[9];
  assign d31p[6]  = dbuf[7]   ~^  dbuf[10];
  assign d31p[7]  = dbuf[8]   ~^  dbuf[11];
  assign d31p[8]  = dbuf[9]   ~^  dbuf[12];
  assign d31p[9]  = dbuf[10]  ~^  dbuf[13];
  assign d31p[10] = dbuf[11]  ~^  dbuf[14];
  assign d31p[11] = dbuf[12]  ~^  dbuf[15];
  assign d31p[12] = dbuf[13]  ~^  dbuf[16];
  assign d31p[13] = dbuf[14]  ~^  dbuf[17];
  assign d31p[14] = dbuf[15]  ~^  dbuf[18];
  assign d31p[15] = dbuf[16]  ~^  dbuf[19];

  assign dnext[15:0]  = dbuf[31:16];

  assign dnext[31:16] = (PSEL == 2'b00) ? d07p :
                        (PSEL == 2'b01) ? d15p :
                        (PSEL == 2'b10) ? d23p : d31p;


  assign QOUT = (INVD == 1'b0) ? dbuf[15:0] : ~dbuf[15:0];

  always @(posedge CLK or posedge RST) begin

    if (RST) begin

      dbuf <= 0;

    end else begin

      dbuf <= dnext;

    end

  end

endmodule



module par16qrbs (
  output reg  ERR,
  input  wire [15:0] DIN,
  input  wire [1:0] PSEL,
  input  wire INVD,
  input  wire RST,
  input  wire CLK,
  input  wire PCC,
  input  wire PSS
);

  wire [15:0] d07p;
  wire [15:0] d15p;
  wire [15:0] d23p;
  wire [15:0] d31p;

  wire [31:0] dnext;

  reg  [31:0] dbuf;

  wire [15:0]  dchek;

  wire [15:0]  DINU;
  assign DINU = (INVD == 1'b0) ? DIN : ~DIN;

  assign d07p[0]  = dbuf[25]  ~^  dbuf[26];
  assign d07p[1]  = dbuf[26]  ~^  dbuf[27];
  assign d07p[2]  = dbuf[27]  ~^  dbuf[28];
  assign d07p[3]  = dbuf[28]  ~^  dbuf[29];
  assign d07p[4]  = dbuf[29]  ~^  dbuf[30];
  assign d07p[5]  = dbuf[30]  ~^  dbuf[31];
  assign d07p[6]  = dbuf[31]  ~^  d07p[0];
  assign d07p[7]  = d07p[0]   ~^  d07p[1];
  assign d07p[8]  = d07p[1]   ~^  d07p[2];
  assign d07p[9]  = d07p[2]   ~^  d07p[3];
  assign d07p[10] = d07p[3]   ~^  d07p[4];
  assign d07p[11] = d07p[4]   ~^  d07p[5];
  assign d07p[12] = d07p[5]   ~^  d07p[6];
  assign d07p[13] = d07p[6]   ~^  d07p[7];
  assign d07p[14] = d07p[7]   ~^  d07p[8];
  assign d07p[15] = d07p[8]   ~^  d07p[9];

  assign d15p[0]  = dbuf[17]  ~^  dbuf[18];
  assign d15p[1]  = dbuf[18]  ~^  dbuf[19];
  assign d15p[2]  = dbuf[19]  ~^  dbuf[20];
  assign d15p[3]  = dbuf[20]  ~^  dbuf[21];
  assign d15p[4]  = dbuf[21]  ~^  dbuf[22];
  assign d15p[5]  = dbuf[22]  ~^  dbuf[23];
  assign d15p[6]  = dbuf[23]  ~^  dbuf[24];
  assign d15p[7]  = dbuf[24]  ~^  dbuf[25];
  assign d15p[8]  = dbuf[25]  ~^  dbuf[26];
  assign d15p[9]  = dbuf[26]  ~^  dbuf[27];
  assign d15p[10] = dbuf[27]  ~^  dbuf[28];
  assign d15p[11] = dbuf[28]  ~^  dbuf[29];
  assign d15p[12] = dbuf[29]  ~^  dbuf[30];
  assign d15p[13] = dbuf[30]  ~^  dbuf[31];
  assign d15p[14] = dbuf[31]  ~^  d15p[0];
  assign d15p[15] = d15p[0]   ~^  d15p[1];

  assign d23p[0]  = dbuf[9]   ~^  dbuf[14];
  assign d23p[1]  = dbuf[10]  ~^  dbuf[15];
  assign d23p[2]  = dbuf[11]  ~^  dbuf[16];
  assign d23p[3]  = dbuf[12]  ~^  dbuf[17];
  assign d23p[4]  = dbuf[13]  ~^  dbuf[18];
  assign d23p[5]  = dbuf[14]  ~^  dbuf[19];
  assign d23p[6]  = dbuf[15]  ~^  dbuf[20];
  assign d23p[7]  = dbuf[16]  ~^  dbuf[21];
  assign d23p[8]  = dbuf[17]  ~^  dbuf[22];
  assign d23p[9]  = dbuf[18]  ~^  dbuf[23];
  assign d23p[10] = dbuf[19]  ~^  dbuf[24];
  assign d23p[11] = dbuf[20]  ~^  dbuf[25];
  assign d23p[12] = dbuf[21]  ~^  dbuf[26];
  assign d23p[13] = dbuf[22]  ~^  dbuf[27];
  assign d23p[14] = dbuf[23]  ~^  dbuf[28];
  assign d23p[15] = dbuf[24]  ~^  dbuf[29];

  assign d31p[0]  = dbuf[1]   ~^  dbuf[4];
  assign d31p[1]  = dbuf[2]   ~^  dbuf[5];
  assign d31p[2]  = dbuf[3]   ~^  dbuf[6];
  assign d31p[3]  = dbuf[4]   ~^  dbuf[7];
  assign d31p[4]  = dbuf[5]   ~^  dbuf[8];
  assign d31p[5]  = dbuf[6]   ~^  dbuf[9];
  assign d31p[6]  = dbuf[7]   ~^  dbuf[10];
  assign d31p[7]  = dbuf[8]   ~^  dbuf[11];
  assign d31p[8]  = dbuf[9]   ~^  dbuf[12];
  assign d31p[9]  = dbuf[10]  ~^  dbuf[13];
  assign d31p[10] = dbuf[11]  ~^  dbuf[14];
  assign d31p[11] = dbuf[12]  ~^  dbuf[15];
  assign d31p[12] = dbuf[13]  ~^  dbuf[16];
  assign d31p[13] = dbuf[14]  ~^  dbuf[17];
  assign d31p[14] = dbuf[15]  ~^  dbuf[18];
  assign d31p[15] = dbuf[16]  ~^  dbuf[19];

  assign dnext[15:0]  = dbuf[31:16];
  assign dnext[31:16] = DINU;

  assign dchek        = (PSEL == 2'b00) ? d07p :
                        (PSEL == 2'b01) ? d15p :
                        (PSEL == 2'b10) ? d23p : d31p;


  always @(posedge CLK or posedge RST) begin
    if (RST) begin
      dbuf <= 0;
    end else begin
      dbuf <= dnext;
    end
  end

  always @(posedge CLK) begin
    ERR  <= (dchek == DINU) ? 0 : 1;
  end

endmodule

