// This file : ploopALN.v
// Rev 2.0   : 05/10/2024


`timescale 1ns / 1ps



// BC = 17C or 283

module BCdet (
  output wire Y,
  input  wire [9:0] DVAL
);

  assign Y = ((DVAL == 10'b0101111100) || (DVAL == 10'b1010000011)) ? 1 : 0;
endmodule

module aln20 (
  output reg  [19:0] DATO,
  output reg  arcv,
  output reg  aerr,
  input  wire [19:0] DATI,
  input  wire CLK,
  input  wire PCC,
  input  wire PSS
);

  reg  [39:0] a;

  wire [19:0] b;

  wire [19:0] sh;
  reg  [19:0] shd;
  reg  [19:0] shdd;

  wire shdet;


  assign b = (shd[19]) ? a[38:19] :
             (shd[18]) ? a[37:18] :
             (shd[17]) ? a[36:17] :
             (shd[16]) ? a[35:16] :
             (shd[15]) ? a[34:15] :
             (shd[14]) ? a[33:14] :
             (shd[13]) ? a[32:13] :
             (shd[12]) ? a[31:12] :
             (shd[11]) ? a[30:11] :
             (shd[10]) ? a[29:10] :
             (shd[9])  ? a[28:9]  :
             (shd[8])  ? a[27:8]  :
             (shd[7])  ? a[26:7]  :
             (shd[6])  ? a[25:6]  :
             (shd[5])  ? a[24:5]  :
             (shd[4])  ? a[23:4]  :
             (shd[3])  ? a[22:3]  :
             (shd[2])  ? a[21:2]  :
             (shd[1])  ? a[20:1]  : a[19:0];


  BCdet xbc19 (.DVAL(a[28:19]), .Y(sh[19]));
  BCdet xbc18 (.DVAL(a[27:18]), .Y(sh[18]));
  BCdet xbc17 (.DVAL(a[26:17]), .Y(sh[17]));
  BCdet xbc16 (.DVAL(a[25:16]), .Y(sh[16]));
  BCdet xbc15 (.DVAL(a[24:15]), .Y(sh[15]));
  BCdet xbc14 (.DVAL(a[23:14]), .Y(sh[14]));
  BCdet xbc13 (.DVAL(a[22:13]), .Y(sh[13]));
  BCdet xbc12 (.DVAL(a[21:12]), .Y(sh[12]));
  BCdet xbc11 (.DVAL(a[20:11]), .Y(sh[11]));
  BCdet xbc10 (.DVAL(a[19:10]), .Y(sh[10]));
  BCdet xbc09 (.DVAL(a[18:9]),  .Y(sh[9]));
  BCdet xbc08 (.DVAL(a[17:8]),  .Y(sh[8]));
  BCdet xbc07 (.DVAL(a[16:7]),  .Y(sh[7]));
  BCdet xbc06 (.DVAL(a[15:6]),  .Y(sh[6]));
  BCdet xbc05 (.DVAL(a[14:5]),  .Y(sh[5]));
  BCdet xbc04 (.DVAL(a[13:4]),  .Y(sh[4]));
  BCdet xbc03 (.DVAL(a[12:3]),  .Y(sh[3]));
  BCdet xbc02 (.DVAL(a[11:2]),  .Y(sh[2]));
  BCdet xbc01 (.DVAL(a[10:1]),  .Y(sh[1]));
  BCdet xbc00 (.DVAL(a[9:0]),   .Y(sh[0]));
  

  assign shdet = sh[19] | sh[18] | sh[17] | sh[16] | sh[15] | sh[14] | sh[13] | sh[12] | sh[11] | sh[10] |
                 sh[9]  | sh[8]  | sh[7]  | sh[6]  | sh[5]  | sh[4]  | sh[3]  | sh[2]  | sh[1]  | sh[0];

  always @(posedge CLK) begin
    if (shdet) begin
      shd  <= sh;
      shdd <= shd;
    end
    a[39:20] <= DATI;
    a[19:0]  <= a[39:20];

    DATO     <= b;
    arcv     <= shdet;
    aerr     <= (shd != shdd);
  end


endmodule




module ploopALN (
  output wire [15:0] ZDAT,
  output wire ZCLK,
  input  wire [15:0] WDAT,
  input  wire WCLK,
  input  wire SCLK,
  input  wire [2:0] GSEL,
  input  wire TX_RST,
  input  wire RX_RST,
  input  wire PCC,
  input  wire PSS
);


  reg ONEVAL = 1;
  reg ZERVAL = 0;




  // * *******************************************
  // * Transmit FIFO
  // * *******************************************

  wire [15:0] tdat;
  wire TX_FULL;
  wire TX_EMPT;

  wire [8:0] datM;
  wire [8:0] datL;


  fifo16 xtfifo (
   .rdata(tdat),
   .wfull(TX_FULL),
   .rempty(TX_EMPT),

   .wdata(WDAT),

   .winc(ONEVAL),
   .wclk(WCLK),
   .wrst(TX_RST),

   .rinc(ONEVAL),
   .rclk(SCLK),
   .rrst(TX_RST)

  );


  assign datM = (TX_EMPT == 1) ? {1'b1, 8'h3C} : {1'b0, tdat[15:8]};
  assign datL = (TX_EMPT == 1) ? {1'b1, 8'hBC} : {1'b0, tdat[7:0]};


  // * *******************************************



  // * *******************************************
  // * Transmit Encoder
  // * *******************************************
  wire [9:0] qdatM;
  wire [9:0] qdatL;
  wire dspMT;
  wire dspLT;

  reg [9:0] edatM;
  reg [9:0] edatL;
  reg edspT;

  encode xencM (
    .datain(datM),
    .dataout(qdatM),
    .dispin(dspLT),
    .dispout(dspMT)
  );

  encode xencL (
    .datain(datL),
    .dataout(qdatL),
    .dispin(edspT),
    .dispout(dspLT)
  );

  always @(posedge SCLK or posedge TX_RST) begin
    if (TX_RST) begin
      edspT <= 0;
    end else begin
      edspT <= dspMT;
    end
  end


  always @(posedge SCLK) begin
    edatM <= qdatM;
    edatL <= qdatL;
  end
  // * *******************************************

  



  reg  [39:0] QF;
  wire [19:0] QFF;
  wire arcv;
  wire aerr;

  always @(posedge SCLK) begin
    QF[39:30] <= edatM;
    QF[29:20] <= edatL;
    QF[19:10] <= QF[39:30];
    QF[9:0]   <= QF[29:20];
  end

  aln20 xaln (
    .DATO(QFF),
    .arcv(arcv),
    .aerr(aerr),
    .DATI(QF[26:7]),
    .CLK(SCLK),
    .PCC(PCC),
    .PSS(PSS)
  );



  // * *******************************************
  // * fclk       = simple delay of SCLK
  // * fdatM[9:0] = simple delay of edatM[9:0]
  // * fdatL[9:0] = simple delay of edatL[9:0]
  // * *******************************************

  wire fclk;
  assign #(0.100) fclk = SCLK;

  wire [9:0] fdatM;
  assign #(0.300) fdatM = QFF[19:10];

  wire [9:0] fdatL;
  assign #(0.300) fdatL = QFF[9:0];







  // * *******************************************



  // * *******************************************
  // * Receive Decoder
  // * *******************************************

  wire [8:0] adatM;
  wire [8:0] adatL;
  wire dspMR;
  wire dspLR;
  wire derrM;
  wire derrL;

  wire codeM;
  wire codeL;

  reg edspR;
  reg ctrM;
  reg ctrL;
  reg [15:0] gdat;

  wire fenb;

  decode xdecM (
    .datain(fdatM),
    .dataout(adatM),
    .dispin(dspLR),
    .dispout(dspMR),
    .disp_err(derrM),
    .code_err(codeM)
  );


  decode xdecL (
    .datain(fdatL),
    .dataout(adatL),
    .dispin(edspR),
    .dispout(dspLR),
    .disp_err(derrL),
    .code_err(codeL)
  );

  always @(posedge fclk or posedge RX_RST) begin
    if (RX_RST) begin
      edspR <= 0;
    end else begin
      edspR <= dspMR;
    end
  end


  always @(posedge fclk) begin
    {ctrM, gdat[15:8]} <= adatM;
    {ctrL, gdat[7:0]}  <= adatL;
  end

  assign fenb = ~ctrM & ~ctrL;

  // * *******************************************


  // * *******************************************
  // * Receive FIFO
  // * *******************************************

  wire [15:0] rdat;
  wire RX_FULL;
  wire RX_EMPT;

  fifo16 xrfifo (
   .rdata(rdat),
   .wfull(RX_FULL),
   .rempty(RX_EMPT),

   .wdata(gdat),

   .winc(fenb),
   .wclk(fclk),
   .wrst(RX_RST),

   .rinc(ONEVAL),
   .rclk(fclk),
   .rrst(RX_RST)

  );


  // * *******************************************


  // * *******************************************
  // * Recoverd Data and Clock --> ZDAT[15:0] ZCLK
  // * *******************************************
  reg [15:0] GDATA;
  reg GDLK;


  always @(posedge fclk) begin
    GDATA <= (RX_EMPT == 0) ? rdat : GDATA;
  end

  always @(posedge fclk or posedge RX_RST) begin
    if (RX_RST) begin
      GDLK  <= 0;
    end
    else begin
      GDLK  <= GDLK ~^ RX_EMPT;
    end
  end


  wire GDLKD;

  wire GDLKDLY0;
  wire GDLKDLY1;
  wire GDLKDLY2;
  wire GDLKDLY3;
  wire GDLKDLY4;
  wire GDLKDLY5;
  wire GDLKDLY6;
  wire GDLKDLY7;

  assign #1.500 GDLK0 = GDLK;
  assign #1.500 GDLK1 = GDLK0;
  assign #1.500 GDLK2 = GDLK1;
  assign #1.500 GDLK3 = GDLK2;
  assign #1.500 GDLK4 = GDLK3;
  assign #1.500 GDLK5 = GDLK4;
  assign #1.500 GDLK6 = GDLK5;
  assign #1.500 GDLK7 = GDLK6;

  assign GDLKD = (GSEL == 3'd7) ? GDLK7 :
                 (GSEL == 3'd6) ? GDLK6 :
                 (GSEL == 3'd5) ? GDLK5 :
                 (GSEL == 3'd4) ? GDLK4 :
                 (GSEL == 3'd3) ? GDLK3 :
                 (GSEL == 3'd2) ? GDLK2 :
                 (GSEL == 3'd1) ? GDLK1 : GDLK0;

  assign rclk = GDLK ~^ GDLKD;

  assign #(0.300) ZDAT = GDATA;
  assign #(0.100) ZCLK = rclk;

  // * *******************************************



endmodule

